#!/usr/bin/env python

import sys
import os
import os.path
from optparse import OptionParser

def readInResultFile(filename = None):
    if not filename or not os.path.isfile(filename):
	return dict()

    query_results = dict()
    f = open(filename,'r')
    data = f.readlines()
    f.close()

    for line in data:
	org_query, rewritten_query, hits, search_time = line.split(";")
	hits = int(hits)
	search_time = int(search_time)

	nr_query_terms = len(org_query.split(" "))

	query_results[org_query] = dict(q_len=nr_query_terms,
		rewritten_query=rewritten_query,
		hits = hits,
		search_time = search_time
		)

    return query_results

def getStopwords():
    f = open("stop_words.txt", "r")
    data = f.readlines()
    stopwords = []
    for i in data:
	if i.startswith("#"):
	    continue
	stopwords.append(i.replace("\n",""))
    f.close()
    return stopwords

def fetchPlotsBasedOnQuery(results, length, withStopwords=False):
    """
	Same as 'def fetchPlots()', but will use every single value!
    """
    if not isinstance(results,dict):
	return False
    if not isinstance(length, int):
	raise ValueError("Invalid input value for parameter 'length'. Got %s" % str(length))

    stopwords = getStopwords()

    # creates a fresh dict of search times
    newDict = {}
    for q in results.keys():
	q_len = int(results[q].get('q_len'))
	if q_len is not length:
	    continue

	query_words = q.split(" ")
	gotStopword = False
	for w in query_words:
	    if w in stopwords:
		gotStopword = True
	    break
	if withStopwords:
	    if not gotStopword:
		continue
	else:
	    if gotStopword:
		continue


	search_time = results[q].get('search_time')

	# we want to construct a vector to hold 
	# the search_time along with the # of times it occurs
	if newDict.get(search_time):
	    newDict[search_time] += 1
	else:
	    newDict[search_time] = 1
    sorted_keys = newDict.keys()
    sorted_keys.sort()
    for i in sorted_keys:
	print "%i\t%i" % (i, newDict[i])

def fetchPlots(results, length):
    """
	Filter out only queries of size == length.
    """
    if not isinstance(results,dict):
	return False
    if not isinstance(length, int):
	raise ValueError("Invalid input value for parameter 'length'. Got %s" % str(length))

    # creates a fresh dict of search times
    newDict = {}
    # creates the buckets to hold the values!
    buckets = {}
    for i in xrange(0,15000,50):
	buckets[i] = {'value':0, 'count':0}

    for q in results.keys():
	q_len = int(results[q].get('q_len'))
	if q_len is not length:
	    continue

	search_time = results[q].get('search_time')

	if newDict.get(search_time):
	    newDict[search_time] += 1
	else:
	    newDict[search_time] = 1
   
    sorted_buckets_keys = buckets.keys()
    sorted_buckets_keys.sort()
    for i in newDict.keys():
	tmp = 0
	for x in sorted_buckets_keys:
	    if i < x:
		#buckets[x] += newDict[i]
		buckets[x]['value'] += newDict[i]
		buckets[x]['count'] += 1
		tmp = x
		#print "%i put in bucket[%i]. Bucket is now: %i" % (i,x,buckets[x])
		break
    # normalize the numbers in each bucket. (divide by 50, which is the bucket size)
    for i in sorted_buckets_keys:
	#print "%i\t%i\t%i\t\tavg:%i" % (i, buckets[i]['value'], buckets[i]['count'], (buckets[i]['value'] and buckets[i]['value']/buckets[i]['count'] or 0) )
	if not buckets[i]['value']:
	    continue
	print "%i\t%i" % (i, buckets[i]['value'])

	## Tar aa printer ut snittet til hver "bucket". Dette for aa hindre altfor store verdier.
	#print "%i\t%i" % (i, (buckets[i]['value'] and buckets[i]['value']/buckets[i]['count'] or 0) )


def queriesWithStopwordsOfVariableLength(results, length, reverted=False):
    # ugly function name!
    if not isinstance(results, dict):
	return False
    if not isinstance(length, int):
	raise ValueError("Invalid input value for parameter 'length'. Got %s" % str(length))

    phrase_types = dict()
    stopwords = getStopwords()

    interesting_queries = []
    
    for q in results.keys():
	q_len = int(results[q].get('q_len'))

	# want only queries of length 2
	if q_len is not length:
	    continue

	# if stopword in query, append to interesting_queries[]
	words_in_query = q.split(" ")

	if not reverted:
	    for term in words_in_query:
	        if term in stopwords:
		    interesting_queries.append( q )
		    break
	else:
	    stopword_query = False
	    for term in words_in_query:
		if term in stopwords:
		    stopword_query = True
		    break

	    if not stopword_query:
		interesting_queries.append( q )
    return interesting_queries

def queriesWithoutStopwordsOfVariableLength(results, length):
    """
	This is just a wrapper for 'def queriesWithStopwordsOfVariableLength()', but 
	just flipping the last argument in function call.
    """
    return queriesWithStopwordsOfVariableLength(results, length, True)


def countDifferntPhraseTypes(results):
    if not isinstance(results, dict):
	return False

    phrase_types = dict()
    phrase_avg_times = dict()

    stopwords = getStopwords()
    print "=========== Got %i stopwords in memory! ============\n" % len(stopwords)

    for q in results.keys():
	q_len = results[q].get('q_len')
	if not q_len in phrase_types.keys():
	    phrase_types[q_len] = 1
	else:
	    phrase_types[q_len] += 1
	
	search_time = results[q].get('search_time')

	stopwords_in_q = len([i for i in q.split(" ") if i in stopwords])

	if not q_len in phrase_avg_times.keys():
	    phrase_avg_times[q_len] = [search_time, 1, stopwords_in_q]
	else:
	    stopwords_in_q = phrase_avg_times[q_len][2]+stopwords_in_q
	    phrase_avg_times[q_len] = [phrase_avg_times[q_len][0]+search_time, phrase_avg_times[q_len][1]+1, stopwords_in_q]
	
   
    print "Summary of phrase lengths and occurences in query result file.."
    total_phrases = 0
    for i in phrase_types.keys():
	tmp_avg_time = phrase_avg_times[i][0]
	count = float(phrase_avg_times[i][1])

	avg_time = float(tmp_avg_time / count)
	avg_stopwords_in_q = float(phrase_avg_times[i][2] / count)
	total_phrases += phrase_types[i]

	print "phrase length: %i  Occurences: %s Average_search_time: %s Average_nr_of_stopwords_in_query: %f" % \
	    (i, str(phrase_types[i]).rjust(4), str(avg_time).rjust(14), avg_stopwords_in_q)
    print "Total nr of phrase queries with hits: %i" % total_phrases

    return (phrase_types, phrase_avg_times)


if __name__ == "__main__":
    usage = "usage: %prog [options]"

    parser = OptionParser(usage=usage)
    parser.add_option("-s", "--statistics", action="store_true", dest="statistics", default=False, help="Will output the statistics of the input result file.")
    parser.add_option("-o", "--fetchPlotBasedOnQuery", action="store_true", dest="plots_based_on_query", default=False, help="Will print out plot values \
	    to be used in Gnuplot graphs. NOTE that you may specify with -l or -r to have the plots based on queries with or without stopwords. Default without.")
    parser.add_option("-p", "--fetchPlots", action="store_true", dest="plots", default=False, help="Will print out plot values to be used in Gnuplot graphs.")
    parser.add_option("-l", "--listOfstopwords", action="store_true", dest="listofstopwords", default=False, \
	    help="Will output a list of phrase queries which is of variable length (default 2) and contains stopwords in it.")
    parser.add_option("-i", "--length", action="store", dest="phrase_length", default=2, help="Length of queries which is to be searched for. Only usefull \
	    when used together with -l.")
    parser.add_option("-r", "--listOfNoneStopwords", action="store_true", dest="listofnonestopwords", default=False, \
	    help="Will output a list of phrase queries which is of variable lenght (default 2) and does NOT contain stopwords in it.")
    parser.add_option("-f", "--filename", action="store", type="string", dest="filename", help="This is required, since this is the result file.")
    (options, args) = parser.parse_args()

    if len(args) < 2:
	if not options.filename:
	    parser.error("Need the result file to be able to do stuff..")
	    sys.exit(0)

	parsedResults = readInResultFile(options.filename)
	if options.statistics:
	    countDifferntPhraseTypes(parsedResults)
	
	if options.plots:
	    if not options.phrase_length or not int(options.phrase_length):
		length = 2
	    else:
		length = int(options.phrase_length)
	    fetchPlots(parsedResults, length)

	if options.plots_based_on_query:
	    if not options.phrase_length or not int(options.phrase_length):
		length = 2
	    else:
		length = int(options.phrase_length)

	    if options.listofstopwords:
		fetchPlotsBasedOnQuery(parsedResults, length, True)
	    else:
		fetchPlotsBasedOnQuery(parsedResults, length, False)
	    # will exit here, so the section below won't get executed due to 
	    # 'options.listofstopwords' and 'options.listofnonestopwords'.
	    sys.exit(0)
	
	if options.listofstopwords or options.listofnonestopwords:
	    if not options.phrase_length or not int(options.phrase_length):
		length = 2
	    else:
		length = int(options.phrase_length)

	    if options.listofstopwords:
		queries = queriesWithStopwordsOfVariableLength(parsedResults, length)
		print "Queries of length %i with stopwords in it: %i" % (length, len(queries))
	    else:
		queries = queriesWithoutStopwordsOfVariableLength(parsedResults, length)
		print "Queries of length %i without stopwords in it: %i" % (length, len(queries))
	    x = 1
	    for i in queries:
		print ':'.join([str(x),i])
		x += 1
