package baldr;

import java.io.IOException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.ListIterator;

import org.apache.lucene.analysis.Token;
import org.apache.lucene.analysis.TokenFilter;
import org.apache.lucene.analysis.TokenStream;
import org.apache.lucene.index.Term;
import org.apache.lucene.search.PhraseQuery;

public class CommonGrams {
	private static final char SEPARATOR = '-';
	private static final String KEY = CommonGrams.class.getName();
	private HashMap<String, HashSet> commonTerms = new HashMap<String, HashSet>();
	
	public CommonGrams() {
		
	}
	
	private static class Filter extends TokenFilter {
		private HashSet common;
		private Token previouse;
		private LinkedList gramQueue = new LinkedList();
		private LinkedList nextQueue = new LinkedList();
		private StringBuffer buffer = new StringBuffer();
		
		public Filter(TokenStream input, HashSet common) {
			super(input);
			this.common = common;
		}
		
		public Token next() throws IOException {
			if (gramQueue.size() != 0) {
				return (Token) gramQueue.removeFirst();
			}
			
			/* fetch from nextQueue if any, else from input TokenStream */
			final Token token = popNext();
			
			if(token == null) {
				return null;
			}
			
			gramQueue.add(token);
			
			ListIterator i = nextQueue.listIterator();
			Token gram = token;
			while(isCommon(gram)) {
				if (previouse != null && !isCommon(previouse)) { // queue prev gram first
					gramQueue.addFirst( gramToken(previouse, gram) );
				}
				Token next  = peekNext(i);
				if(next == null) {
					break;
				}
				
				gram = gramToken(gram, next);
				gramQueue.addLast(gram);
			}
			
			previouse = token;
			return (Token) gramQueue.removeFirst();
		}
		
		private boolean isCommon(Token token) {
			return common != null && !common.contains(token.termText());
		}
		
		private Token popNext() throws IOException {
			if (nextQueue.size() > 0) {
				return (Token) nextQueue.removeFirst();
			} else {
				return input.next();
			}
		}
		
		private Token peekNext(ListIterator i) throws IOException {
			if(i.hasNext()) {
				Token next = input.next();
				if (next == null) {
					return null;
				}
				i.add(next);
				i.previous();
			}
			return (Token) i.next();
		}
		
		
		/** Construct a compound token. */
	    private Token gramToken(Token first, Token second) {
	      buffer.setLength(0);
	      buffer.append(first.termText());
	      buffer.append(SEPARATOR);
	      buffer.append(second.termText());
	      Token result = new Token(buffer.toString(),
	                               first.startOffset(), second.endOffset(),
	                               "gram");
	      result.setPositionIncrement(0);
	      return result;
	    }
	    
	    // private static class end
	}
	
	/** Construct a token filter that inserts n-grams for common terms.  For use
	   * while indexing documents.  */
	public TokenFilter getFilter(TokenStream ts, String field) {
		return new Filter(ts, (HashSet)commonTerms.get(field));
	}
	
/** Utility to convert an array of Query.Terms into a token stream. */
	/*
	private static class ArrayTokens extends TokenStream {
		private Term[] terms;
		private int index;

		public ArrayTokens(Phrase phrase) {
			this.terms = phrase.getTerms();
		}

		public Token next() {
			if (index == terms.length)
				return null;
			else
				return new Token(terms[index].toString(), index, ++index);
		}
	}*/
}
