package baldr;

import java.io.File;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.SimpleAnalyzer;

/**
 * Contains basic variables which is crucial for the search system to function.
 * @author fellingh
 */
public class Definitions {
	
	/**
	 * Decides the path to which to Lucene index is to be stored
	 */
	//public static final String indexDir = "/home/fellingh/skole/gov2_index_bigram";
	
	public static final String userDir = System.getProperty("user.dir");
	
    @Deprecated
	public static final String indexDir = "/home/fellingh/skole/gov2_index_bigram";
    public static final String indexDir_unigrams = "/home/fellingh/skole/gov2_unigrams";
    public static final String indexDir_full_bigram = "/home/fellingh/skole/gov2_full_bigram";
    public static final String indexDir_bigram_0 = "/home/fellingh/skole/gov2_bigram_0";
    public static final String indexDir_bigram_10 = "/home/fellingh/skole/gov2_bigram_10";
    public static final String indexDir_bigram_100 = "/home/fellingh/skole/gov2_bigram_100";
    
    public static final String indexDir_testindex = "/home/fellingh/skole/gov2_testindex";

	
	/**
	 * Decides the path to the documents which is to be indexed. This path is crucial
	 * to the TrecParser and Gov2Parser.
	 * @see TrecParser Gov2Parser 
	 */
	//public static final String baseDir = "/home/fellingh/skole/trec_gov2";
	public static final String baseDir = "/home/fellingh/skole/trec_gov2";
	
	/**
	 * The file which specifies which TREC GOV2 files we will parse!
	 * <b>Note:</b> that all the file paths is relative, not absolute!
	 */
	public static final File trecFile = new File(userDir+File.separator+"trecFiles.txt");
    
    /**
     * The file with all the queries that we shall test aginst our index. The 
     * format of the file is one query pr line; 'id:query' --> '1:prison', '2:to be or'.
     */
    public static final File queryFile = new File("/home/fellingh/workspace/MasterExperiment/gov2_queries.txt");
    //public static final File queryFile = new File("/home/fellingh/workspace/MasterExperiment/search_results_111stopwordqueries/gov2_queries_length_2_stopwords_bigram10");
    //public static final File queryFile = new File("/home/fellingh/workspace/MasterExperiment/search_results_444stopwordqueries/gov2_queries_length_2_stopwords_bigram100");
    
    /**
     * The file with phrase queries that has been verified to produce hits. This file will also contain phrase queries
     * which has been manually added. This is to force the search engine to look through the hole index, and therefor 
     * use some resources.
     */
    public static final File phraseQueryFile = new File("/home/fellingh/workspace/MasterExperiment/gov2_phrase_queries.txt");

    /**
     * 
     */
    public static final File searchResultFile = new File("/home/fellingh/workspace/MasterExperiment/search_results.txt");
    
    /**
     * Global variabel who decides wheter to enable debugging!
     */
    public static final boolean DEBUG = true;
    
    /**
     * A simple analyzer which simple breaks the input stream into simple tokens. 
     */
    public static final Analyzer unigramAnalyzer = new SimpleAnalyzer();
    
    /**
     * An analyzer which creates bigrams of every set of two words. <br/>
     * F.ex: input: "to be or not to be".<br/>
     * Output: "to be", "be or", "or not", "not to", "to be".
     * 
     * <br/><br/>
     * One important thing to notice it that this analyzer by default also creates unigrams for all words.
     */
    public static final Analyzer fullBigramAnalyzer = new DocumentAnalyzer(DocumentAnalyzer.Filters.NGramFilter);
    
    /**
     * An analyzer which creates bigrams of word pairs where the firstword 
     * is defined as a stopword. <br/>
     * <b>Note:</b> this analyzer requires the usage of a stopword file. If none stopwords 
     * is defined in the file, then none bigrams will be created, only unigrams.
     * 
     * <br/><br/>
     * One important thing to notice it that this analyzer by default also creates unigrams for every <b>non-stopword</b>. 
     */
    public static final Analyzer bigramAnalyzer = new DocumentAnalyzer(DocumentAnalyzer.Filters.PhraseFilter3);
    
}
