package baldr;

import baldr.search.Searcher;


/*
 * Type: Singelton
 * Purpose:
 * 	Functions like a mini-db to keep collections of object 
 *  references and to make sure that there's only _one_ 
 *  instance of each object (whenever this is of importance). 
 */

public class Hub {
	private static Hub instance;
	private Indexer index;
	private Searcher search;
	private ParseBadTrec badTrec;
	
	private Hub() {	
	}
	
	public Indexer getIndexer() {
		if (this.index == null) {
			this.index = new Indexer();
		}
		return this.index;
	}
	
	public Searcher getSearcher() {
		if (this.search == null) {
			this.search = new Searcher();
		}
		return this.search;
	}
	
	public ParseBadTrec getParseBadTrec() {
		if (this.badTrec == null) {
			this.badTrec = new ParseBadTrec();
		}
		return this.badTrec;
	}
	
	public static Hub getInstance() {
		if(instance == null) {
			instance = new Hub();
		}
		return instance;
	}
}
