
package baldr;

import java.util.HashMap;

import org.apache.lucene.analysis.Analyzer;

import baldr.search.Searcher;


/*
 * Purpose:
 * 	Will index a TREC collection with the Lucene search library. 
 *  Will first create a standard index, and evaluate different queries, 
 *  mostly phrase queries.
 *  
 *  Then, will index the same collection with Lucene search library, but 
 *  with the use of n-grams of size 2 (two words indexed as a single semantic unit).
 *  And, so evaluate the same queries performed above. 
 *  
 *  In both the query evaluations I will compare the results (semantic and time
 *  results).
 */

public class Main {
	public static boolean recreateTrecFiles = false;
	public static boolean index = false;
	public static boolean search = true;
	public static Hub hub;

	public static void main(String[] args) {
		/* This bools under here represents switches 
		 * which can be 'flipped' to execute a command.
		 * 
		 *  NOTE: the 'recreateTreFiles' is only to be run 
		 *  when/if you have standard uncompresse TREC files 
		 *  to be converted into a semi-SGML format.
		 *  */
			
		hub = Hub.getInstance();
		String searchprop = System.getProperty("willie.search");
		String indexprop = System.getProperty("willie.index");
		
		if ( (index || !indexprop.equals(" ")) && (searchprop == null || !searchprop.equals("true"))) {
			startIndex();
		} else if (search) {
			startSearcher();
		} /*else if (recreateTrecFiles) {
			startParsingBadTrec();
		} */
	}
	
	public static void startIndex() {
		System.out.println("Will now start indexing!");
		
		String indexType = System.getProperty("willie.index");
		if(indexType == null || indexType.equals("unknown")) {
			System.out.println("Must define indexing property 'willie.index'.");
			System.exit(0);
		}
		
		String indexDir="";
		Analyzer analyzer = null;
		
		if(indexType.equals("unigrams")) {
			indexDir = Definitions.indexDir_unigrams;
			analyzer = Definitions.unigramAnalyzer;
			
		} else if (indexType.equals("bigrams_full")) {
			indexDir = Definitions.indexDir_full_bigram;
			analyzer = Definitions.fullBigramAnalyzer;
			
		} else if (indexType.equals("bigrams_0")) {
			indexDir = Definitions.indexDir_bigram_0;
			analyzer = Definitions.bigramAnalyzer;
			
		} else if (indexType.equals("bigrams_10")) {
			indexDir = Definitions.indexDir_bigram_10;
			analyzer = Definitions.bigramAnalyzer;
			
		} else if (indexType.equals("bigrams_100")) {
			indexDir = Definitions.indexDir_bigram_100;
			analyzer = Definitions.bigramAnalyzer;	
			
		} else if (indexType.equals("testindex")) {
			indexDir = Definitions.indexDir_testindex;
			analyzer = Definitions.bigramAnalyzer;
		}
		
		if(indexDir.equals("")) {
			System.out.println("Will quit. Could not find indexDir");
			System.exit(0);
		} else if(analyzer == null) {
			System.out.println("Will quit. Could not find analyzer!");
			System.exit(0);
		}
		
		Indexer indexer = hub.getIndexer();
		indexer.startIndex(indexDir, analyzer);
	}
	
	public static void startSearcher() {
		System.out.println("Searching..");
		
		String indexType = System.getProperty("willie.index");
		if(indexType == null || indexType.equals("unknown")) {
			System.out.println("Must define indexing property 'willie.index'.");
			System.exit(0);
		}
		
		String indexDir="";
		Analyzer analyzer = null;
		
		if(indexType.equals("unigrams")) {
			indexDir = Definitions.indexDir_unigrams;
			analyzer = Definitions.unigramAnalyzer;
			
		} else if (indexType.equals("bigrams_full")) {
			indexDir = Definitions.indexDir_full_bigram;
			analyzer = Definitions.fullBigramAnalyzer;
			
		} else if (indexType.equals("bigrams_0")) {
			indexDir = Definitions.indexDir_bigram_0;
			analyzer = Definitions.bigramAnalyzer;
			
		} else if (indexType.equals("bigrams_10")) {
			indexDir = Definitions.indexDir_bigram_10;
			analyzer = Definitions.bigramAnalyzer;
			
		} else if (indexType.equals("bigrams_100")) {
			indexDir = Definitions.indexDir_bigram_100;
			analyzer = Definitions.bigramAnalyzer;	
		} else {
			System.out.println("Could set indexDir and / or analyzer. Will quit.");
			System.exit(0);
		}
		
		// Omitting the Hub here!
		//Searcher searcher = hub.getSearcher(indexDir, analyzer);
		Searcher searcher = new Searcher(indexDir, analyzer, indexType);
		searcher.initThreads();
	}
	
	public static void startParsingBadTrec() {
		System.out.println("This function is deprecated!");
		System.exit(0);
		
		System.out.println("Parsing Trec files, and stores them!");
		ParseBadTrec badTrec = hub.getParseBadTrec();
		badTrec.start();
	}

}
