package baldr;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Vector;

import org.apache.commons.lang.StringUtils;



public class TrecParser implements Runnable {
    /*
     * Tar inn en "path" som input, og tar s  henter ut alle filer og 
     * parser de som TREC filer.
     */
    File dir;
    File[] filer;
    ArrayList<Trec96Document> documents = new ArrayList();
    public boolean parsing = true;
    public boolean collect = false;
    private boolean debug = false;
    private int maxNumFiles = 30;
    private int numFiles = 0;
    private int numDocs = 0;
    private int maxNumDocs = 300;
    private String dirName;
    private Indexer indexer;
    //private ngramsTest.gov2parser.TRECParser testParser;
    
    public TrecParser(String dirName, Indexer in) {
        this.dirName = dirName;
        this.indexer = in;
    }
    
    public void parseGOV2Docs(String[] files) {
    	
    }
    
    public void run() {
        dir = new File(this.dirName);
        if (! dir.exists()) {
            System.out.println(this.dirName+" finnes ikke.");
            return ;
        }
        
        filer = dir.listFiles();
        BufferedReader in;
        String line;
        Vector<String> tdoc = new Vector<String>();
      
        for(File f : filer) {
            /*
             * Hent hver fil og XML-parse dem!
             * 
             * For hver <doc> lag et nytt Trec96Document!
             * Samle alle Trec96Document, og indekser dem!
             */
        	
            //System.out.println("Leser fil: "+f.getAbsolutePath());
        	if(f.isDirectory()) {
        		System.out.println("\t"+f.getName()+" is a directory. Skipping!");
        		continue;
        	}
        	
            try {
                in = new BufferedReader(new FileReader(f));
                
                while((line = in.readLine()) != null) {
                    tdoc.add(line);
                }
                in.close();
                
            } catch(IOException e) {
                System.out.println("Fikk ikke lest: "+f.getAbsolutePath());
                e.printStackTrace();
            }

            indexTrecFile(tdoc);
            tdoc.clear();
            numFiles++;
            
        }
        
        this.parsing = false;
        this.indexer.doneParsing();
    }

    
    private void indexTrecFile(Vector data) {
        
        String tmpString = "";
        //StringBuilder tmpString2 = new StringBuilder()
        Trec96Document tmpTrec;
        boolean aDoc = false;
        String docno="", headline="", date="", text = "";
        Enumeration<String> vdoc = data.elements();
        
        while(vdoc.hasMoreElements()) {
            if (numDocs >= maxNumDocs) {
            	/* Sends them to the indexer, then flushes the internal documents arraylist. */
            	this.indexer.hasDocs(documents);
            	documents.clear();
            	numDocs = 0;
            }
            	
        	try {
                tmpString = (String) vdoc.nextElement();
                if(StringUtils.contains(tmpString, "<doc>")) {
                	
                }
                
                if(tmpString.startsWith("<doc>")) {
                	aDoc = true;
                	if(StringUtils.contains(tmpString, "<docno>")) {
                		docno = StringUtils.substringBetween(tmpString, "<docno>", "</docno>");
                	} else {
                		System.out.println("Weird.. Could locate a <docno>! "+tmpString);
                	}
                } else if(StringUtils.contains(tmpString, "<date>")) {
                	date = StringUtils.substringBetween(tmpString, "<date>", "</date>");
                } else if(StringUtils.contains(tmpString, "<headline>")) {
                	headline = StringUtils.substringBetween(tmpString, "<headline>", "</headline>");
                } else if(StringUtils.contains(tmpString, "<content>")) {
                	text = StringUtils.substringBetween(tmpString, "<content>", "</content>");
                } else if(tmpString.startsWith("</doc>")) {
                	/* Check if we got a complete doc! */
                	try {
                		if(docno.equals("")) throw new Exception();
                		else if(headline.equals("")) throw new Exception();
                		else if(date.equals("")) throw new Exception();
                		else if(text.equals("")) throw new Exception();
                		else {
                			tmpTrec = new Trec96Document(docno, headline, date, text);
                			documents.add(tmpTrec);
                			numDocs++;
                			
                			/* Cleaning up! */
                			tmpTrec = null;
                			aDoc = false;
                			docno = "";
                			headline = "";
                			date = "";
                			text = "";
                		}
                	} catch(Exception e) {
                		e.printStackTrace();
                	}
                } else {
                	System.out.println("Something very wrong happend! "+tmpString);
                }
            } catch(Exception e) {
            	e.printStackTrace();
            }
        }
        
        //if (numDocs >= maxNumDocs) {
        	/* Sends them to the indexer, then flushes the internal documents arraylist. */
    	this.indexer.hasDocs(documents);
    	documents.clear();
    	numDocs = 0;
        //}
    }
    
    
    public ArrayList<Trec96Document> getDocuments() {
        return this.documents;
    }
}
