package baldr.search;

import it.unimi.dsi.mg4j.util.MutableString;

import java.io.IOException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Set;
import java.util.Vector;
import java.util.concurrent.Callable;

import org.apache.commons.collections.buffer.BoundedFifoBuffer;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.index.Term;
import org.apache.lucene.queryParser.MultiFieldQueryParser;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.queryParser.QueryParser;
import org.apache.lucene.search.Hit;
import org.apache.lucene.search.Hits;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.PhraseQuery;
import org.apache.lucene.search.Query;


/**
 * A thread which will query the search engine for the whole partition of 
 * queries that the Searcher gave me!
 * @author asbjorn
 * @see Searcher
 */
public class SearchRobot implements Runnable {
	private IndexSearcher isearcher;
	private final BoundedFifoBuffer queries;
	private HashSet<String[]> newQueries;
	private HashMap<String, Vector<Object>> searchResults = new HashMap<String, Vector<Object>>(); // 10000 items
	
	private Logger logger = Logger.getLogger(this.getClass());
	private Analyzer analyzer;
	private Searcher parent;
	private int thread_nr;
	
    public SearchRobot(IndexSearcher isearcher, int nr, final BoundedFifoBuffer queryBuffer, Analyzer analyzer, final Searcher parent) {
        this.isearcher = isearcher;
        this.queries = queryBuffer;
        this.analyzer = analyzer;
        this.parent = parent;
        this.thread_nr = nr;
        
        logger.setLevel(Level.INFO);
        logger.debug("Starting..!");
    }
    
    public HashSet<String[]> call() throws Exception {
        return newQueries;
    }
    
    
    public void run() {
    	logger.debug("run...");
    	
    	PhraseQuery query = null;
    	Query q;
    	//QueryParser qp = new QueryParser("text", this.analyzer);
    	MultiFieldQueryParser qp = new MultiFieldQueryParser(new String[] {"headline","text"}, this.analyzer);
    	Hits hits;
    	Hit hit;
    	Document doc;
    	MutableString fieldString = new MutableString();
    	
    	String[] terms = null;
    	String phraseQuery = null;
    	Vector<Object> extendedResultInfo = new Vector<Object>(3);
    	
    	while(true) {
    		synchronized (queries) {
    			if(!queries.isEmpty()) {
    				//terms = ((String)queries.remove()).split(" ");
    				phraseQuery = (String) queries.remove();
    			} else {
    				break;
    			}
    		}
    		
    		try {
    			//q = QueryParser.parse
    			q = qp.parse("\""+phraseQuery+"\"");
    			/*
    			query = new PhraseQuery();
    			for(String t : terms ) {
        			query.add(new Term("text", t));
        		}
    			*/
    			long start = System.currentTimeMillis();
				hits = isearcher.search(q);
				long stop = System.currentTimeMillis();
				if(hits.length() > 0) {
					extendedResultInfo.add(q.toString());
					extendedResultInfo.add(hits.length());
					extendedResultInfo.add((stop-start));
					logger.debug("Got hit. #"+hits.length()+" hits from phrase query: \""+q.toString()+"\". Used: "+(stop-start)+" ms. "+extendedResultInfo.toString());
				} else {
					logger.debug(Thread.currentThread().getName()+" didn't get anything from query: "+q.toString());
				}
				searchResults.put(phraseQuery, (Vector<Object>)extendedResultInfo.clone());
				extendedResultInfo.clear();
    		} catch(IOException e) {
    			e.printStackTrace();
    		} catch(ParseException pe) {
    			pe.printStackTrace();
    		}
    	}
    	
    	logger.debug("SearchRobot "+Thread.currentThread().getName()+" done..");
    	parent.stateArray[thread_nr] = true;
    }
    
    public HashMap<String, Vector<Object>> getResults() {
    	return this.searchResults;
    }
}
