package baldr.search;

import it.unimi.dsi.mg4j.util.MutableString;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Vector;
import java.util.concurrent.Executor;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.Future;
import java.util.concurrent.FutureTask;

import org.apache.commons.collections.buffer.BoundedFifoBuffer;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.SimpleAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.index.Term;
import org.apache.lucene.queryParser.MultiFieldQueryParser;
import org.apache.lucene.queryParser.ParseException;
import org.apache.lucene.queryParser.QueryParser;
import org.apache.lucene.search.Hit;
import org.apache.lucene.search.HitIterator;
import org.apache.lucene.search.Hits;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.PhraseQuery;
import org.apache.lucene.search.Query;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.FSDirectory;

import baldr.Definitions;
import baldr.DocumentAnalyzer;



public class Searcher {
	/*
	 * The following is to be in this component:
	 * 	1. A interpreter where you can choose which analyzer (and index) 
	 * there is to be used. And to enter any type of query and retrieve the results 
	 * in the command line.
	 */
    public static Directory directory;
    public static IndexSearcher isearcher;
    private Logger logger = Logger.getLogger(this.getClass());
    private HashMap<Integer, String> testQueries = new HashMap<Integer, String>(50000);
    private BoundedFifoBuffer queryBuffer = new BoundedFifoBuffer(50000);
    private ExecutorService executor = Executors.newFixedThreadPool(3);
    private static Analyzer analyzer;
    //private static Analyzer analyzer = new SimpleAnalyzer();
    private String[] fields = new String[] {"headline", "text"};
    private int threads = 2;
    public boolean[] stateArray;
    private String indexType;
    
    public Searcher(String indexDir, Analyzer analyzer) {
    	this.analyzer = analyzer;
        if(Definitions.DEBUG) {
            logger.setLevel(Level.DEBUG);
        }
        long timeStart = System.currentTimeMillis();
        try {
            directory = FSDirectory.getDirectory(indexDir);
            isearcher = new IndexSearcher(directory);
            logger.debug("Index successfully loaded! "+indexDir);
        } catch (IOException e) {
            e.printStackTrace();
            logger.error("Could not setup IndexSearcher to read index: "+indexDir);
        }
        long timeStop = System.currentTimeMillis();
        logger.debug("Tok "+(timeStop-timeStart)+" millisekunder!");
        /* Setting up ThreadPool! */
        
        //executor.execute(new SearchRobot());
        readQueryFile();
        /*try {
        	//testQuery();
        	//HashMap<Integer, String> phraseQueries = findUsableQueries();
        	//logger.debug("Out of "+testQueries.size()+" queries, "+phraseQueries.size()+" was usable as phrase queries.");
        	
        	//writeQueriesToFile(Definitions.phraseQueryFile, phraseQueries);
        	//logger.debug("Has written: "+phraseQueries.size()+" queries to file: "+Definitions.phraseQueryFile);
        	
        } catch(Exception e) {
        	e.printStackTrace();
        }
        close(); */
    }
    
    /**
     * The purpose of this constructor is to support the usage result_filename modification. That is, the string supplied 
     * in the paramtere indexType will be appended at the end of the filename for {@link Definitions.searchResultFile}.<br/>
     * 
     * It will make it simpler to just initiate batch jobs, and then later on collect the search result files based on which 
     * index was used and such.
     * @param indexDir
     * @param analyzer
     * @param indexType
     */
    public Searcher(String indexDir, Analyzer analyzer, String indexType) {
    	this(indexDir, analyzer);
    	this.indexType = indexType;
    }
    
    /**
     * When a empty constructor is called, then we use by default the {@link Definitions.indexDir_unigrams} index. <br/>
     * If a string with another index path is passed as argument, then that index path is used.
     *
     */
    public Searcher() {
        this(Definitions.indexDir_unigrams, new SimpleAnalyzer());
    }
    
    
    /**
     * This method initalizes and starts a new fixed threadpool of size {@link Searcher#indexType}. 
     * Places two instances of {@link SearchRobot} in the threadpool, which each performes queries 
     * agains the index {@link Searcher#isearcher}. <br/>
     * 
     * After completion of the queries, the results are collected, and then written to a file.
     *
     */
    public void initThreads() {
    	ExecutorService executor = Executors.newFixedThreadPool(threads);
    	
    	//Thread[] threadPool = new Thread[threads];
    	SearchRobot[] robots = new SearchRobot[threads];
    	stateArray = new boolean[threads];
    	Future[] futures = new Future[threads];
    	HashSet<String[]> result = null;
    	
    	for(int i=0;i<threads;i++) {
    		robots[i] = new SearchRobot(isearcher, i, this.queryBuffer, this.analyzer, this);
    		//threadPool[i] = new Thread(robots[i]);
    		futures[i] = new FutureTask( robots[i], result);
    	}
    	
    	logger.debug("Has created searching threads. Trying to start them..");
    	
    	
    	for(int i=0;i<threads;i++) {
    		executor.execute(robots[i]);
    		stateArray[i] = false;
    	}
    	
    	outer: while(true) {
    		for(int t=0;t<threads;t++) {
    			if(futures[t].isDone()) {
    				stateArray[t] = true;
    			}
    		}
    		
    		inner: for(int t=0;t<threads;t++){
    			if(!stateArray[t]) {
    				try {
    					//logger.debug("sleeping 2 sek. futures[0]: "+futures[0].isCancelled()+" futures[1]: "+futures[1].isCancelled());
    					Thread.currentThread().sleep(3000);	
    				} catch(InterruptedException e) {
    					e.printStackTrace();
    				}
    				continue outer;
    			}
    		}
    		// if we get here, that means all Executors has finished. Time to collect data!
    		break;
    	}
    	executor.shutdown();
    	logger.debug("Querying is finished. Will now collect data....");
    	
    	HashMap<String, Vector<Object>> result1 = robots[0].getResults();
    	HashMap<String, Vector<Object>> result2 = robots[1].getResults();
    	result1.putAll(result2);
		try {
			writeResultsToFile(result1);
		} catch (IOException e) {
			// TODO: handle exception
			e.printStackTrace();
		}
		logger.debug("Done with experiment!");
		
		// need to close the index after using it.
		close();
    }
    
    private void writeResultsToFile(final HashMap<String, Vector<Object>> results) throws IOException {
    	File resultFile;
    	if (indexType != null) {
    		resultFile = new File(Definitions.searchResultFile.getAbsolutePath()+"_"+indexType);
    	} else {
    		resultFile = Definitions.searchResultFile;
    	}

    	BufferedWriter bw = new BufferedWriter(new FileWriter(resultFile));
    	int hits;
    	long time;
    	String rewrittenQuery;
    	Vector<Object> tmpVector; 
    	for(String phraseQuery : results.keySet()) {
    		tmpVector = results.get(phraseQuery);

    		if(tmpVector.size() > 0) {
    			rewrittenQuery = (String) tmpVector.remove(0);
    			hits = (Integer) tmpVector.remove(0);
    			time = (Long) tmpVector.remove(0);
    			bw.write(phraseQuery+";"+rewrittenQuery+";"+hits+";"+time+"\n");
    			//logger.debug("query: "+phraseQuery+" hits: "+hits+" time: "+time+" ms rewrittenQuery: "+rewrittenQuery);
    		} 
    	}
    	bw.close();
    }
    
    private void readQueryFile() {
    	int x=0;
        try {
            if(!Definitions.queryFile.isFile()) {
                throw new IOException(Definitions.queryFile.toString()+" was not a file!");
            }
            
            /* Read in and parse each line of text into queryId and query! */
            BufferedReader in = new BufferedReader(new FileReader(Definitions.queryFile));
            String tmp, query;
            String[] o;
            int queryId;
            
            while( (tmp=in.readLine()) != null ) {
            	if(tmp.startsWith("#")) continue;
                o = tmp.split(":");  
                query = o[1].trim();
                
                // Only phrases of length > 1
                if(query.split(" ").length > 1) {
                	//testQueries.put(queryId, query);
                	queryBuffer.add(query);
                }
                x++;
            }
            in.close();
        } catch(IOException e) {
            e.printStackTrace();
            logger.error(": Could not add queries from query file!");
            return;
        }
        logger.debug("Added "+queryBuffer.size()+" of "+x+" possible queries!");
    }
    
    private HashMap<Integer, String> findUsableQueries() {
    	HashMap<Integer, String> usableQueries = new HashMap<Integer, String>();
    	PhraseQuery query = null;
    	Hits hits;
    	Hit hit;
    	Document doc;
    	MutableString fieldString = new MutableString();
    	
    	for(Integer i : testQueries.keySet()){ 
    		
    		query = new PhraseQuery();
    		String terms[] = testQueries.get(i).split(" ");
    		
    		for(String t : terms ) {
    			query.add(new Term("text", t));
    		}
    		
    		try {
    			long start = System.currentTimeMillis();
				hits = isearcher.search(query);
				long stop = System.currentTimeMillis();
				
				if(hits.length() > 0) {
					logger.debug("Got hit. #"+hits.length()+" hits from phrase query: \""+query.toString()+"\". Used: "+(stop-start)+" ms");
					usableQueries.put(i, testQueries.get(i));
				} else {
					logger.debug("None hits for phrase query: "+query.toString()+". Used: "+(stop-start)+" ms");
				}
			} catch (IOException e) {
				e.printStackTrace();
			} 
    		
    	}
    	return usableQueries;
    }
    
    private void writeQueriesToFile(File outFile, HashMap<Integer, String> queries) {
    	try {
    		BufferedWriter bw = new BufferedWriter(new FileWriter(outFile));
    		
    		for(Integer i : queries.keySet()) {
    			bw.write(i+":"+queries.get(i)+"\n");
    		}
    		
    		bw.close();
    	} catch(IOException e) {
    		e.printStackTrace();
    	}
    }
    
    private void testQuery() throws Exception {
        int maxNr = 10, x=0;
        int docId;
        PhraseQuery query = null;
        Hits hits;
        Hit hit;
        
        Document doc;
        MutableString fieldString = new MutableString();
        
        for(Integer i : testQueries.keySet()) {
            if(x==maxNr) break;
            x++;
            
            try {
                //query = parser.parse( testQueries.get(i));
            	//query = new PhraseQuery();
            	query = new PhraseQuery();
            	String[] terms = testQueries.get(i).split(" ");;
            	for(String t : terms) {
            		query.add(new Term("text", t));
            	}
            	
            } catch(Exception e) {
                e.printStackTrace();
            }
            hits = isearcher.search(query);
            if(hits.length() > 0) {
            	logger.debug("Got hit. #"+hits.length()+" hits from phrase query: \""+query.toString()+"\"");
            }
            
            /*
            for(int j=0;j<hits.length();j++) {
            	doc = hits.doc(j);
            	List<Field> test = doc.getFields();
            	for(Field t : test) {
            		fieldString.append(t.toString()+" ");
            	}
            	logger.debug("   Score: "+hits.score(j)+"  Doc nr: "+doc.getField("docno").stringValue()+" headline: "+doc.getField("headline").stringValue()+" fields: "+fieldString);
            	
            	break;
            } */
        }
        logger.debug(": Done searching!");
    }
    
    private void close() {
        try {
            isearcher.close();
        } catch (IOException e) {
            logger.error("Could not close the IndexSearcher.");
        }
    }
}
