package baldr.utility;

import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import it.unimi.dsi.mg4j.document.CompositeDocumentFactory;
import it.unimi.dsi.mg4j.document.Document;
import it.unimi.dsi.mg4j.document.DocumentFactory;
import it.unimi.dsi.mg4j.document.DocumentIterator;
import it.unimi.dsi.mg4j.document.HtmlDocumentFactory;
import it.unimi.dsi.mg4j.document.PropertyBasedDocumentFactory;
import it.unimi.dsi.mg4j.document.TRECDocumentCollection;
import it.unimi.dsi.mg4j.document.TRECHeaderDocumentFactory;
import it.unimi.dsi.mg4j.io.FastBufferedReader;
import it.unimi.dsi.mg4j.util.MutableString;
import it.unimi.dsi.mg4j.util.Properties;

import org.apache.commons.collections.buffer.BoundedFifoBuffer;
import org.apache.log4j.Level;
import org.apache.log4j.Logger;
import org.apache.lucene.document.Field;

import baldr.Definitions;
import baldr.Gov2Document;
import baldr.Gov2Document2;


public class SingleTRECDocReader {
	private Properties p = new Properties();
    private TRECHeaderDocumentFactory thd = new TRECHeaderDocumentFactory();
    private HtmlDocumentFactory hdf;
    private DocumentFactory docFac;
    private TRECDocumentCollection d;
    private Logger logger = Logger.getLogger(this.getClass());
    private Gov2Document2 newDoc;
    private int counter = 0;
    private ArrayList<String> files = new ArrayList<String>();
    private HashMap<String, Object> results = new HashMap<String, Object>();
    private BoundedFifoBuffer buffer = new BoundedFifoBuffer(1200);
    
    public static void main(String[] args) {
    	
    	SingleTRECDocReader ttdr = new SingleTRECDocReader();
    	
    	long start = System.currentTimeMillis();
    	ttdr.init();
    	long stop = System.currentTimeMillis();
    	
    	
    	System.out.println("Took "+(stop-start)+" ms to initialize AND read in TREC file! Let the action begin!");
    	ttdr.results.put("init_time", (stop-start));
    	
    	ttdr.parse();
    }
    
    public void init() {
    	logger.setLevel(Level.DEBUG);
    	try {
        	p.addProperty(PropertyBasedDocumentFactory.MetadataKeys.ENCODING, new String("ISO-8859-1"));
        	hdf = new HtmlDocumentFactory( p );
        	DocumentFactory[] facArray = {thd, hdf};
        	docFac = CompositeDocumentFactory.getFactory(facArray, new String[] {"text","title","anchor"});
        }catch(Exception e) {
        	e.printStackTrace();
        }
        
        files.add("/home/fellingh/skole/trec_gov2/GX004/83");
    	logger.debug("Will now parse "+files.size()+" TREC files.");
    	
    	
    }
    
    private void parse() {
    	long start = System.currentTimeMillis();
    	
    	indexFile( files.subList(0, 1)); /* extracts a subList of how many files I want to parse at once */
    	
    	long stop = System.currentTimeMillis();
    	results.put("time_to_complete_parse", (stop-start));
    	results.put("number of documents", counter);
    	
    	for(String key : results.keySet()) {
    		logger.debug("key: "+key+" Value: "+results.get(key));
    	}
    }
    
    private void indexFile(List<String> files) {
    	long start = System.currentTimeMillis();
    	MutableString tmp = new MutableString();
    	String filesString;
    	String[] filesArray;
    	
    	for(String fil : files) {
    		tmp.append(fil+" ");
    	}
    	filesString = tmp.toString();
    	filesArray = filesString.split(" ");
    	long stop = System.currentTimeMillis();
    	
    	//if(results.get("time_convert_arraylist_string[]"))
    	long prevTime = 0;
    	if( results.containsKey("time_convert_arraylist_to_string[]") ) {
    		prevTime = (Long) results.get("time_convert_arraylist_to_string[]");
    	}
    	long result;
    	if(prevTime == 0) {
    		result = (stop-start);
    	} else {
    		result = ((stop-start)+prevTime)/2;
    	}
    	results.put("time_convert_arraylist_to_string[]", result );
    	indexFile(filesArray);
    }
    
    private void indexFile(String[] gov2File) {
    	long start = System.currentTimeMillis();
    	
    	Document dt;
    	DocumentIterator it;
    	MutableString title = new MutableString();
    	MutableString content = new MutableString();
    	FastBufferedReader r;
    	MutableString parsedContent = new MutableString();
    	
    	int c;
    	char l, prev;
    	
    	try {
    		// Buffer size:
    		// 8192 bytes
    		// 16384 bytes
        	d = new TRECDocumentCollection(gov2File, docFac, 16384, false);        	
        	it = d.iterator();

        	
        	do {
        		dt = it.nextDocument();
        		
        		// Very important to break out if there is no more docs!
        		if (dt==null) break;
        		
        		parsedContent.setLength(0);
        		r = (FastBufferedReader) dt.content(0);
        		r.readLine(content);
        		prev = ' ';
        		while( (c = r.read()) != -1) {
        			l = (char) c;
        			if (prev == ' ' && l == ' ') continue; // vil ikke ha repeterende mellomrom!
        			if (l == '\n' || l == '\r' || l=='\t') {
        				continue;
        			}
        			parsedContent.append(l);
        			prev = l;
        		}
        		
        		r = (FastBufferedReader) dt.content(1);
        		r.readLine(title);
        		//logger.debug("Title: "+title);
        		
        		// Now, the content is ready, so we continue to read on!
        		counter++;
        		
        		//System.out.println("title: "+title);
        		//System.out.println("content: "+parsedContent.toString());
        		
        		//newDoc = new Gov2Document2(dt.title().toString(), title.toString(), dt.uri().toString(), parsedContent.toString());
        		newDoc = new Gov2Document2();
        		newDoc.setHeadline(title.toString());
        		newDoc.setUri(dt.uri().toString());
        		newDoc.setText(parsedContent.toString());
        		
        		buffer.add(newDoc);
        		
        	} while(true); // continues until the iterator returns null!
        	it.close();
        	
        	writeParsedFile();
        	logger.debug("We're done parsing the docs!");
        } catch(Exception e) {
        	e.printStackTrace();
        }
        long stop = System.currentTimeMillis();
        long result;
        long prevTime = 0;
        if(results.containsKey("time_to_parse_10_files")) {
        	prevTime = (Long) results.get("time_to_parse_10_files");
        }
        
        if(prevTime == 0) {
        	result = (stop-start);
        } else {
        	result = ((stop-start)+prevTime)/2;
        }
        results.put("time_to_parse_10_files", result);
    }
    
    private void fetchFiles(File trecFile) {	
		if(!trecFile.isFile()) {
			logger.error("Could not open: "+trecFile.getPath()+". Need this file to determine ");
			return;
		}
		File tmpFile;
		
		try {
			BufferedReader in = new BufferedReader(new FileReader(trecFile));
			String t = "";
			while( (t = in.readLine()) != null) {
				files.add(Definitions.baseDir+File.separator+t.replace("\n", ""));
			}
		} catch(IOException e) {
			e.printStackTrace();
		}
	}
    
    private void writeParsedFile() {
    	File outFile = new File("parsedTRECFile.txt");
    	try {
    		BufferedWriter bw = new BufferedWriter(new FileWriter(outFile));
    		Gov2Document2 doc;
    		org.apache.lucene.document.Document d;
    		Field text, headline, uri;
    		MutableString output = new MutableString();
    		while(!buffer.isEmpty()) {
    			doc = (Gov2Document2)buffer.remove();
    			d = doc.document();
    			
    			text = d.getField("text");
    			headline = d.getField("headline");
    			uri = d.getField("uri");
    			
    			output.setLength(0); /* nullstiller string */
    			output.append("uri: "+uri.stringValue()+"\n");
    			output.append("headline: "+headline.stringValue()+"\n");
    			output.append("text: "+text.stringValue()+"\n");
    			
    			bw.write(output.toString());
    		}
    		bw.close();
    	}catch(IOException e) {
    		e.printStackTrace();
    	}
    }
}
