package baldr.utility;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import it.unimi.dsi.mg4j.document.CompositeDocumentFactory;
import it.unimi.dsi.mg4j.document.Document;
import it.unimi.dsi.mg4j.document.DocumentFactory;
import it.unimi.dsi.mg4j.document.DocumentIterator;
import it.unimi.dsi.mg4j.document.HtmlDocumentFactory;
import it.unimi.dsi.mg4j.document.PropertyBasedDocumentFactory;
import it.unimi.dsi.mg4j.document.TRECDocumentCollection;
import it.unimi.dsi.mg4j.document.TRECHeaderDocumentFactory;
import it.unimi.dsi.mg4j.io.FastBufferedReader;
import it.unimi.dsi.mg4j.util.MutableString;
import it.unimi.dsi.mg4j.util.Properties;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import baldr.Definitions;
import baldr.Gov2Document;


public class TestTRECDocReader {
	private Properties p = new Properties();
    private TRECHeaderDocumentFactory thd = new TRECHeaderDocumentFactory();
    private HtmlDocumentFactory hdf;
    private DocumentFactory docFac;
    private TRECDocumentCollection d;
    private Logger logger = Logger.getLogger(this.getClass());
    private Gov2Document newDoc;
    private int counter = 0;
    private int documentsToParse = 1000;
    private ArrayList<String> files = new ArrayList<String>();
    private HashMap<String, Object> results = new HashMap<String, Object>();
    
    public static void main(String[] args) {
    	
    	TestTRECDocReader ttdr = new TestTRECDocReader();
    	
    	long start = System.currentTimeMillis();
    	ttdr.init();
    	long stop = System.currentTimeMillis();
    	
    	
    	System.out.println("Took "+(stop-start)+" ms to initialize AND read in TREC file! Let the action begin!");
    	ttdr.results.put("init_time", (stop-start));
    	
    	ttdr.parse();
    }
    
    public void init() {
    	logger.setLevel(Level.DEBUG);
    	try {
        	p.addProperty(PropertyBasedDocumentFactory.MetadataKeys.ENCODING, new String("ISO-8859-1"));
        	hdf = new HtmlDocumentFactory( p );
        	DocumentFactory[] facArray = {thd, hdf};
        	docFac = CompositeDocumentFactory.getFactory(facArray, new String[] {"text","title","anchor"});
        }catch(Exception e) {
        	e.printStackTrace();
        }
        
        fetchFiles(Definitions.trecFile);
    	logger.debug("Will now parse TREC files.");
    	
    	
    }
    
    private void parse() {
    	/* parses 10 files at a time! */
    	long start = System.currentTimeMillis();
    	
    	//int threshold = 200;
	if (documentsToParse < files.size()) {
	    indexFile( files.subList(0, documentsToParse));
	} else {
	    indexFile( files );
	    documentsToParse = files.size();
	}
    	/*
    	int x=0, last = 0;
    	for(int i=0; i<files.size(); i++){
    		//if (i==threshold) break;
    		if(x==10) {
    			indexFile( files.subList(last, i) );
    			last = i;
    			x=0;
    		}
    		x++;
    	} */
    	
    	long stop = System.currentTimeMillis();
    	results.put("time_to_complete_parse", (stop-start));
    	results.put("number of documents", counter);
    	
    	for(String key : results.keySet()) {
    		logger.debug("key: "+key+" Value: "+results.get(key));
    	}
    }
    
    private void indexFile(List<String> files) {
    	long start = System.currentTimeMillis();
    	MutableString tmp = new MutableString();
    	String filesString;
    	String[] filesArray;
    	
    	for(String fil : files) {
    		tmp.append(fil+" ");
    	}
    	filesString = tmp.toString();
    	filesArray = filesString.split(" ");
    	long stop = System.currentTimeMillis();
    	
    	//if(results.get("time_convert_arraylist_string[]"))
    	long prevTime = 0;
    	if( results.containsKey("time_convert_arraylist_to_string[]") ) {
    		prevTime = (Long) results.get("time_convert_arraylist_to_string[]");
    	}
    	long result;
    	if(prevTime == 0) {
    		result = (stop-start);
    	} else {
    		result = ((stop-start)+prevTime)/2;
    	}
    	results.put("time_convert_arraylist_to_string[]", result );
    	indexFile(filesArray);
    }
    
    private void indexFile(String[] gov2File) {
    	long start = System.currentTimeMillis();
    	
    	Document dt;
    	DocumentIterator it;
    	MutableString title = new MutableString();
    	MutableString content = new MutableString();
    	FastBufferedReader r;
    	MutableString parsedContent = new MutableString();
    	
    	int c;
    	char l, prev;
	long stop = 0;
    	
    	try {
    		// Buffer size:
		// 1024 bytes 
		// 2048 bytes
		// 4096 bytes
    		// 8192 bytes
    		// 16384 bytes
		// 32768 bytes
		// 65536 bytes
		int bufferSize = 8192;
		logger.debug("Using buffer size: "+bufferSize);
        	d = new TRECDocumentCollection(gov2File, docFac, bufferSize, false);        	
        	it = d.iterator();
        	
        	do {
        		//parsedContent = new MutableString();
        		parsedContent.setLength(0);
        		dt = it.nextDocument();
        		
        		// Very important to break out if there is no more docs!
        		if (dt==null) break;
        		
        		r = (FastBufferedReader) dt.content(0);
        		r.readLine(content);
        		prev = ' ';
        		while( (c = r.read()) != -1) {
        			l = (char) c;
        			if (prev == ' ' && l == ' ') continue; // vil ikke ha repeterende mellomrom!
        			if (l == '\n' || l == '\r' || l=='\t') {
        				continue;
        			}
        			parsedContent.append(l);
        			prev = l;
        		}
        		
        		r = (FastBufferedReader) dt.content(1);
        		r.readLine(title);
        		//logger.debug("Title: "+title);
        		
        		// Now, the content is ready, so we continue to read on!
        		counter++;
        		
        		//newDoc = new Gov2Document(dt.title().toString(), title.toString(), dt.uri().toString(), parsedContent.toString());
        		
        	} while(true); // continues until the iterator returns null!
		stop = System.currentTimeMillis();
        	it.close();
        	logger.debug("We're done parsing the docs!");
        } catch(Exception e) {
        	e.printStackTrace();
        }
        long result;
        long prevTime = 0;
        if(results.containsKey("time_to_parse_"+documentsToParse+"_files")) {
        	prevTime = (Long) results.get("time_to_parse_"+documentsToParse+"_files");
        }
        
        if(prevTime == 0) {
        	result = (stop-start);
        } else {
        	result = ((stop-start)+prevTime)/2;
        }
        results.put("time_to_parse_"+documentsToParse+"_files", result);
    }
    
    private void fetchFiles(File trecFile) {	
		if(!trecFile.isFile()) {
			logger.error("Could not open: "+trecFile.getPath()+". Need this file to determine ");
			return;
		}
		File tmpFile;
		
		try {
			BufferedReader in = new BufferedReader(new FileReader(trecFile));
			String t = "";
			while( (t = in.readLine()) != null) {
				files.add(Definitions.baseDir+File.separator+t.replace("\n", ""));
			}
		} catch(IOException e) {
			e.printStackTrace();
		}
		
	}
}
